# encoding: utf-8
class Admin::NewsController < OrbitAdminController
  require 'rubyXL'
  include Admin::NewsHelper
  before_action ->(module_app = @app_title) { set_variables module_app }
  before_action :set_news_bulletin, only: [:edit, :destroy]
  before_action :load_access_level

  def initialize
    super
    @app_title = "news"
  end

  def index
    @tags = @module_app.tags
    @categories = @module_app.categories.enabled
    @filter_fields = filter_fields(@categories, @tags)
    @table_fields = [:status, :category, :title, :start_date, :end_date, :image, "news.unit", :view_count]

    if !params[:sort].blank?
      if params[:sort] == 'start_date'
        sort = {:postdate=>params[:order]}
      elsif params[:sort] == 'end_date'
        sort = {:deadline=>params[:order]}
      else
        sort = {params[:sort].to_sym=>params[:order]}
      end
    else
     sort = {:postdate=>"desc"}
    end

    
    @news_bulletin = NewsBulletin.where(:is_preview.in=>[false,nil])
                                        .order_by(sort)
                                        .with_categories(filters("category"))
                                        .with_tags(filters("tag"))
                                        .with_status(filters("status"))

    @news_bulletin = search_data(@news_bulletin,[:title]).page(params[:page]).per(10)

    if request.xhr?
      render :partial => "index"
    end
  end

  def new
    @tags = @module_app.tags
    @statuses = []
    @news_bulletin = NewsBulletin.new
    @news_bulletin.email_sentdate = Time.now
    sort = (!params[:sort].blank? ? {params[:sort].to_sym=>params[:order]} : {:id=>"asc"})
    @unit = Unit.all.order_by(sort)
  end

  def create
    if !news_bulletin_params['news_bulletin_links_attributes'].nil?
      news_bulletin_params['news_bulletin_links_attributes'].each do |idx,link|
        news_bulletin_params['news_bulletin_links_attributes'].delete(idx.to_s) if link['url'].blank?
      end
    end
    news_bulletin = NewsBulletin.new(news_bulletin_params)
    news_bulletin.create_user_id = current_user.id
    news_bulletin.update_user_id = current_user.id
    if user_can_approve?
      news_bulletin.approved = true
      news_bulletin.save
    else
      news_bulletin.save
      # Thread.new do 
      #   begin
      #     send_notification_mail_to_managers(news_bulletin,"approval")
      #   rescue e
      #   end
      # end
    end
    # build_email(news_bulletin)
    redirect_to params['referer_url']
  end

  def approve_news_bulletin
    id = params[:id]
    news_bulletin = NewsBulletin.find(id)
    if params["approved"] == "true"
      news_bulletin.approved = true
      news_bulletin.rejected = false
      news_bulletin.reapproval = false
      news_bulletin.save
    else
      news_bulletin.rejected = true
      news_bulletin.reapproval = false
      news_bulletin.rejection_reason = params["reason"]
      news_bulletin.save
      # send_rejection_email(news_bulletin)
    end
    redirect_to "/admin/news"
  end

  def edit
    if can_edit_or_delete?(@news_bulletin)
      sort = (!params[:sort].blank? ? {params[:sort].to_sym=>params[:order]} : {:id=>"asc"})
      @unit = Unit.all.order_by(sort)
      @department = Department.where(:unit_id => @news_bulletin.unit_id)
      @tags = @module_app.tags
      @categories = @module_app.categories.enabled
      @statuses = []
      @news_bulletin.email_sentdate = Time.now if @news_bulletin.email_sent == false
      @selected = nil
    else
      render_401
    end
  end

  def update
    uid = params[:id].split('-').last
    news_bulletin = NewsBulletin.where(:uid=>uid).first rescue nil
    NewsBulletin.where(:copy_id=>news_bulletin.id.to_s).destroy
    news_bulletin.is_edit = true
    news_bulletin.save
    set_approved = news_bulletin.is_preview
    news_bulletin = news_bulletin.get_org_model
    news_bulletin.approved = true if set_approved
    news_bulletin_params[:tags] = news_bulletin_params[:tags].blank? ? [] : news_bulletin_params[:tags]
    news_bulletin_params[:email_member_ids] = news_bulletin_params[:email_member_ids].blank? ? [] : news_bulletin_params[:email_member_ids]

    if !news_bulletin_params['news_bulletin_links_attributes'].nil?
      news_bulletin_params['news_bulletin_links_attributes'].each do |idx,link|
        news_bulletin_params['news_bulletin_links_attributes'].delete(idx.to_s) if link['url'].blank?
      end
    end

    news_bulletin.update_attributes(news_bulletin_params)
    if news_bulletin.rejected
      news_bulletin.reapproval = true
      news_bulletin.save
      # Thread.new do 
      #   begin
      #     send_notification_mail_to_managers(news_bulletin, "reapproval")
      #   rescue e
      #   end
      # end
    end
    news_bulletin.save
    # build_email(news_bulletin)
    redirect_to params['referer_url']
  end

  def destroy
    @news_bulletin.destroy
    redirect_to "/admin/news"
  end

  def delete
    if params[:ids]
      NewsBulletin.any_in(:uid => params[:ids]).destroy_all
    end
    redirect_to "/admin/news"
  end

  def render_404
    render :file => "#{Rails.root}/app/views/errors/404.html", :layout => false, :status => 404, :formats => [:html]
  end

  def import_export
    @thread = Multithread.where(:id=>params[:thread_id]).first if params[:thread_id].present?
  end

  def download_file_from_thread
    @thread = Multithread.where(:id=>params[:id]).first if params[:id].present?
    if @thread && @thread.status[:file]
      send_file(@thread.status[:file],:filename=>@thread.status[:filename])
    else
      render_404
    end
  end

  def excel_format
    respond_to do |format|
      format.xlsx {
        response.headers['Content-Disposition'] = 'attachment; filename="news_import_format.xlsx"'
      }
    end
  end

  def export_excel
    @thread = Multithread.where(:key=>'export_news').first
    update_flag = true
    if @thread.nil?
      @thread = Multithread.create(:key=>'export_news',:status=>{:status=>'Processing'})
    else
      update_flag = false if @thread.status[:status] == 'Processing' && @thread.respond_to?(:updated_at) && (@thread.updated_at > DateTime.now - 1.minute rescue false)
      if update_flag
        @thread.update(:status=>{:status=>'Processing'})
      end
    end
    if update_flag
      @host = Site.first.root_url
      if @host == "http"
        @host = request.protocol + request.host_with_port
      end
      Thread.new do
        begin
          @news = NewsBulletin.where(:is_preview.ne=>true).desc(:id)
          last_updated = [NewsBulletin.max(:updated_at).to_i, Unit.max(:updated_at).to_i, Department.max(:updated_at).to_i].max
          filename = "public/news_export_#{last_updated}.xlsx"
          if File.exist?(filename)
            @thread.update(:status=>{:status=>'finish','finish_percent'=>100,'info'=>I18n.t('announcement.read_from_cache')})
          else
            excel_contents = render_to_string( handlers: [:axlsx], formats: [:xlsx] ,partial: 'export_excel.xlsx' )
            File.open(filename, 'w') do |f|
              f.write excel_contents
            end
          end
          @thread.status[:file] = filename
          @thread.status[:filename] = "news_export_#{DateTime.now.in_time_zone(Time.zone.utc_offset / 3600).strftime('%Y_%m_%d_%H%M')}.xlsx"
          @thread.save
        rescue => e
          @thread.status[:status] = 'error'
          # @thread.status[:info] = [e.to_s, e.backtrace]
          puts [e.to_s, e.backtrace]
          @thread.save
        end
      end
    end
    redirect_to admin_news_import_export_path(:thread_id=>@thread.id.to_s)
  end

  def importnews
    workbook = RubyXL::Parser.parse(params["import_file"].tempfile)
    raw_categories = @module_app.categories.asc(:created_at).to_a.map.with_index{|v, k| [k.to_s,v]}.to_h
    raw_tags = @module_app.tags.asc(:created_at).to_a.map.with_index{|v, k| [k.to_s,v]}.to_h
    categories = raw_categories.clone
    tags = raw_tags.clone
    sheet = workbook[0]
    if sheet.count <= 503
      sheet.each_with_index do |row, i|
        if i == 2
          begin
            cats_text = row.cells[0].value.to_s.sub(/(^|,)\s*Example\s*:.*$/,'')
            cats_keys = cats_text.split('->').map{|s| s.split(',')[-1].strip}[0...-1]
            cats_values = cats_text.split('->')[1..-1].to_a.map{|s| s.strip.sub(/,\s*\d+$/,'')}
            categories_relations = cats_keys.zip(cats_values).to_h
          rescue => e
            categories_relations = {}
          end
          begin
            tags_text = row.cells[1].value.to_s.sub(/(^|,)\s*Example\s*:.*$/,'')
            tags_keys = tags_text.split('->').map{|s| s.split(',')[-1].strip}[0...-1]
            tags_values = tags_text.split('->')[1..-1].to_a.map{|s| s.strip.sub(/,\s*\d+$/,'')}
            tags_relations = tags_keys.zip(tags_values).to_h
          rescue => e
            tags_relations = {}
          end
          if categories_relations.present?
            categories = categories_relations.map do |k, v|
              tmp = raw_categories[k]
              if tmp && tmp.title.strip == v
                [k, tmp]
              else
                tmp = raw_categories.detect{|kk, vv| vv.title.strip == v}
                if tmp.nil?
                  tmp = @module_app.categories.create(:title_translations=> localize_data(v))
                end
                [k, tmp]
              end
            end.to_h
          end
          if tags_relations.present?
            tags = tags_relations.map do |k, v|
              tmp = raw_tags[k]
              if tmp && tmp.name.strip == v
                [k, tmp]
              else
                tmp = raw_tags.detect{|kk, vv| vv.name.strip == v}
                if tmp.nil?
                  tmp = @module_app.tags.create(:name_translations=> localize_data(v))
                end
                [k, tmp]
              end
            end.to_h
          end
        end
        next if i < 3
        v = row.cells.first.value rescue nil
        next if v.blank? #類別為空
        import_this_news(row, categories, tags)
      end
      redirect_to admin_news_index_path
    else
      redirect_to admin_news_index_path(:error => "1")
    end
  end

  def preview
    if params['preview_type'].eql?('edit')
      news_bulletin_data = news_bulletin_params
      org_news_bulletin = NewsBulletin.find(params['news_bulletin_id'])
      news_bulletin = org_news_bulletin.clone
      news_bulletin.generate_uid
      news_bulletin.news_bulletin_files = []
      news_bulletin.news_bulletin_links = []
      if news_bulletin_data['image'].blank?
        news_bulletin.image = org_news_bulletin.image
      end

      if !news_bulletin_data['news_bulletin_files_attributes'].blank?
        news_bulletin_data['news_bulletin_files_attributes'].each do |key, news_bulletin_file|
          next if !news_bulletin_file['_destroy'].blank?
          file = nil
          if news_bulletin_file['id'].blank?
            file = NewsBulletinFile.new(news_bulletin_file)
            file.news_bulletin_id = news_bulletin.id
            file.save
          else
            org_file = NewsBulletinFile.find(news_bulletin_file['id'])
            file = org_file.clone
            file.news_bulletin_id = news_bulletin.id
            file.file = org_file.file
            news_bulletin_file.delete('id')
            news_bulletin_file.delete('_destroy')
            file.update_attributes(news_bulletin_file)
          end

          file.save
          news_bulletin.news_bulletin_files << file
        end
      end

      if !news_bulletin_data['news_bulletin_links_attributes'].blank?
        news_bulletin_data['news_bulletin_links_attributes'].each do |key, news_bulletin_link|
          next if !news_bulletin_link['_destroy'].blank?

          if news_bulletin_link['id'].blank?
            link = NewsBulletinLink.new(news_bulletin_link)
            link.news_bulletin_id = news_bulletin.id
          else
            link = NewsBulletinLink.find(news_bulletin_link['id']).clone
            link.news_bulletin_id = news_bulletin.id
            news_bulletin_link.delete('id')
            news_bulletin_link.delete('_destroy')
            link.update_attributes(news_bulletin_link)
          end

          link.save
          news_bulletin.news_bulletin_links << link
        end
      end

      news_bulletin_data.delete('news_bulletin_files_attributes')
      news_bulletin_data.delete('news_bulletin_links_attributes')
      news_bulletin.update_attributes(news_bulletin_data)
      news_bulletin.copy_id = params['news_bulletin_id']
    else
      news_bulletin = NewsBulletin.new(news_bulletin_params)
    end

    news_bulletin.is_preview = true
    news_bulletin.save
    render :text=>page_for_news_bulletin(news_bulletin)
  end
  def get_preview_action
    news_bulletin = NewsBulletin.find_by(:uid=>params['uid'])
    is_not_edit = (!news_bulletin.is_edit)
    org_bulletin = news_bulletin.get_org_model
    NewsBulletin.where(:copy_id=>org_bulletin.id.to_s).destroy
    if is_not_edit
      news_bulletin.destroy
      render :json=> {:success=>true,:action=>"close"}
    else
      render :json=> {:success=>true,:action=>"redirect",:path=>edit_admin_news_path(:id=>org_bulletin.id)}
    end
  end
  def destroy_preview
    news_bulletin = NewsBulletin.find_by(:uid=>params['uid'])
    if news_bulletin.is_preview
      news_bulletin.destroy
    end
    render :json=>{'destroy'=>news_bulletin.id.to_s}
  end

  def build_email(news_bulletin)
    if news_bulletin.email_sent and !news_bulletin.email_addresses.blank?
      if news_bulletin.email.nil?
        email = Email.new
        email.save
        news_bulletin.email_id = email.id
        news_bulletin.save
      end

      is_sent = news_bulletin.email.is_sent
      is_sent = !params[:resend_mail].eql?("true") if !params[:resend_mail].blank?

      news_bulletin.email.update_attributes(
        :create_user=>current_user,
        :mail_sentdate=>news_bulletin.email_sentdate,
        :module_app=>@module_app,
        :mail_to=>news_bulletin.email_addresses,
        :mail_subject=>news_bulletin.title,
        :template=>'news/email',
        :template_data=>{
          "host" => request.host_with_port,
          "title" => news_bulletin.title,
          "url" => page_for_news_bulletin(news_bulletin)
        },
        :is_sent=>is_sent
      )
    else
      news_bulletin.email.destroy if !news_bulletin.email.nil?
    end
  end

  private

  def set_news_bulletin
    @news_bulletin = NewsBulletin.find(params[:id])
  end

  def news_bulletin_params
    params[:news_bulletin][:email_sent] = params[:news_bulletin][:email_sent].nil? ? 0 : params[:news_bulletin][:email_sent]
    params.require(:news_bulletin).permit!
  end
end
